/***********************************************************
 * $Id: fancysocktransstream.h 1336 2014-10-06 11:49:53Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** stream classes, which encapsulate writing/reading a document from/to a
** transport.
** 
*****************************************************************************/

#ifndef FANCYSOCK_TRANS_STREAM_H
#define FANCYSOCK_TRANS_STREAM_H

#include <iostream>
#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {
  class Map;

  namespace Sock {

    class Transport;

    class FANCYSOCK_API itransstreambuf : public std::streambuf
    {
    public:
      itransstreambuf(const Fancy::ObjPtr<Transport> &t, Fancy::ObjPtr<Map> &headers, int timeout);
      virtual ~itransstreambuf();

    protected:
      int_type underflow ();

    private:
      int fill_buffer();

      Fancy::ObjPtr<Transport> transport;
    };

    class FANCYSOCK_API itransstream : public std::istream
    {
      itransstreambuf itsbuf;
    public:
      itransstream(const Fancy::ObjPtr<Transport> &t, Fancy::ObjPtr<Map> &headers, int timeout);
      virtual ~itransstream();
    };

    class FANCYSOCK_API otransstreambuf : public std::streambuf
    {
    public:
      otransstreambuf(const Fancy::ObjPtr<Transport> &t, const Fancy::ObjPtr<Map> &headers);
      virtual ~otransstreambuf();

    protected:
      int_type overflow (int_type c);
      int sync();

    private:
      int put_buffer();

      Fancy::ObjPtr<Transport> transport;
    };

    class FANCYSOCK_API otransstream : public std::ostream
    {
      otransstreambuf otsbuf;
    public:
      otransstream(const Fancy::ObjPtr<Transport> &t, const Fancy::ObjPtr<Map> &headers);
      ~otransstream();
    };

  }
}

#endif
